// BB8 Code by Jay Luna
// Make Course Project USF Spring 2016

#include <I2Cdev.h>
#include <Wire.h>
#include <MPU6050_6Axis_MotionApps20.h>
#include <helper_3dmath.h>
#include <PID_v1.h>                              //Arduino PID library
//#include <digitalIOPerformance.h>                //library for faster pin R/W
#include <SoftwareSerial.h>

#if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
    #include "Wire.h"
#endif

MPU6050 mpu;

// MPU control/status vars
bool dmpReady = false;  // set true if DMP init was successful
uint8_t mpuIntStatus;   // holds actual interrupt status byte from MPU
uint8_t devStatus;      // return status after each device operation (0 = success, !0 = error)
uint16_t packetSize;    // expected DMP packet size (default is 42 bytes)
uint16_t fifoCount;     // count of all bytes currently in FIFO
uint8_t fifoBuffer[64]; // FIFO storage buffer

// orientation/motion vars
Quaternion q;           // [w, x, y, z]         quaternion container
VectorFloat gravity;    // [x, y, z]            gravity vector
float ypr[3];           // [yaw, pitch, roll]   yaw/pitch/roll container and gravity vector

volatile bool mpuInterrupt = false;     // indicates whether MPU interrupt pin has gone high
void dmpDataReady() {
    mpuInterrupt = true;
}

// BlueTooth to App Communication
#define    STX          0x02
#define    ETX          0x03

SoftwareSerial mySerial(11,12); //TX, RX
byte cmd[8]; 

//motor control
//motor 1
#define enA 10
#define in1 9
#define in2 8
//motor 2
#define enB 5
#define in3 7
#define in4 6

int motor1PWM = 0, motor2PWM = 0; //set motor starting point
int motor1Offset = 60, motor2Offset = 64; //motor offset, when they start to spin
int motor1Start = motor1Offset + 60, motor2Start = motor2Offset + 56; // make sure both motors start at same value

double yaw, pitch, roll;
double deadspot = 0; // where BB8 will be upright

int biasY, biasX, Xleft, Xright, joyY2; // joystick direction variables

double SetpointX, InputX, OutputX; // Variables for PID controller
double SetpointY, InputY, OutputY;

double Setpoint; //desired degree angle for BB8 head

double Kp = 3; // PID contants
double Ki = 0;
double Kd = 0;

PID myPIDX(&InputX, &OutputX, &SetpointX, Kp, Ki, Kd, DIRECT); //X axis PID
PID myPIDY(&InputY, &OutputY, &SetpointY, Kp, Ki, Kd, DIRECT); //Y axis PID

void setup(){
  init_imu(); //start MPU6050
  Serial.begin(57600);
  mySerial.begin(57600);  // 57600 = max value for softserial for bluetooth module   
  while(mySerial.available())  mySerial.read(); // empty RX buffer

  myPIDX.SetMode(AUTOMATIC); // set output limits for PID controller
  myPIDX.SetOutputLimits(-255, 255);
  myPIDX.SetSampleTime(10); // sample time for PID controller
  myPIDY.SetMode(AUTOMATIC);
  myPIDY.SetOutputLimits(-255, 255); 
  myPIDY.SetSampleTime(10);

  SetpointX = 0;
  SetpointY = 0;
  Setpoint = SetpointY;
  
  motorSetup(); //enable motor pins as outputs
}

void loop() {
  getValues(); //read values from imu
  blueTooth(); //read bluetooth
  pidCompute(); //Set PID
  motorControl(); //send signal to motors
}

void init_imu() //turn on MPU6050
{
    // join I2C bus (I2Cdev library doesn't do this automatically)
    #if I2CDEV_IMPLEMENTATION == I2CDEV_ARDUINO_WIRE
        Wire.begin();
        TWBR = 24; // 400kHz I2C clock (200kHz if CPU is 8MHz). Comment this line if having compilation difficulties with TWBR.
    #elif I2CDEV_IMPLEMENTATION == I2CDEV_BUILTIN_FASTWIRE
        Fastwire::setup(400, true);
    #endif
     mpu.initialize();
     devStatus = mpu.dmpInitialize();
     // supply your own gyro offsets here, scaled for min sensitivity
    mpu.setXGyroOffset(-31);
    mpu.setYGyroOffset(-7);
    mpu.setZGyroOffset(11);
    mpu.setXAccelOffset(-600);
    mpu.setYAccelOffset(-578);
    mpu.setZAccelOffset(1226);
    // make sure it worked (returns 0 if so)
    if (devStatus == 0) {
        // turn on the DMP, now that it's ready
        mpu.setDMPEnabled(true);
        // enable Arduino interrupt detection
       // Serial.println(F("Enabling interrupt detection (Arduino external interrupt 0)..."));
        attachInterrupt(0, dmpDataReady, RISING);
        mpuIntStatus = mpu.getIntStatus();
        dmpReady = true;
        // get expected DMP packet size for later comparison
        packetSize = mpu.dmpGetFIFOPacketSize();
    }
}

void getValues() //Read values from MPU6050
{
     // if programming failed, don't try to do anything
    if (!dmpReady) return;

    // wait for MPU interrupt or extra packet(s) available
    while (!mpuInterrupt && fifoCount < packetSize) {
    }
    mpuInterrupt = false;
    mpuIntStatus = mpu.getIntStatus();

    // get current FIFO count
    fifoCount = mpu.getFIFOCount();

    // check for overflow (this should never happen unless our code is too inefficient)
    if ((mpuIntStatus & 0x10) || fifoCount == 1024) {
        // reset so we can continue cleanly
        mpu.resetFIFO();
        Serial.println(F("FIFO overflow!"));
    } else if (mpuIntStatus & 0x02) {
        // wait for correct available data length, should be a VERY short wait
        while (fifoCount < packetSize) fifoCount = mpu.getFIFOCount();

        // read a packet from FIFO
        mpu.getFIFOBytes(fifoBuffer, packetSize);
        
        // track FIFO count here in case there is > 1 packet available
        // (this lets us immediately read more without waiting for an interrupt)
        fifoCount -= packetSize;
            mpu.dmpGetQuaternion(&q, fifoBuffer);
            mpu.dmpGetGravity(&gravity, &q);
            mpu.dmpGetYawPitchRoll(ypr, &q, &gravity);
            
         }
         Serial.print("ypr\t");
         Serial.print(ypr[0] * 180/M_PI);
         Serial.print("\t");
         Serial.print(ypr[1] * 180/M_PI);
         Serial.print("\t");
         Serial.println(ypr[2] * 180/M_PI); 
         yaw = ypr[0]* 180/M_PI;
         pitch = -ypr[1] * 180/M_PI;          //change sign if negative
         roll = ypr[2] * 180/M_PI;
}

void blueTooth(){ // enable bluetooth read
 if(mySerial.available())  {   // data received from smartphone
   delay(2);
   cmd[0] =  mySerial.read();  
   if(cmd[0] == STX)  {
     int i=1;      
     while(mySerial.available())  {
       delay(1);
       cmd[i] = mySerial.read();
       if(cmd[i]>127 || i>7)                 break;     // Communication error
       if((cmd[i]==ETX) && (i==2 || i==7))   break;     // Button or Joystick data
       i++;
     }
     if(i==7){
      getJoystickState(cmd);     // 6 Bytes  ex: < STX "200" "180" ETX >
     }
   }
 } 
}

void getJoystickState(byte data[8])    {
 int joyX = (data[1]-48)*100 + (data[2]-48)*10 + (data[3]-48);       // obtain the Int from the ASCII representation
 int joyY = (data[4]-48)*100 + (data[5]-48)*10 + (data[6]-48);
 joyX = joyX - 200;                                                  // Offset to avoid
 joyY = joyY - 200;                                                  // transmitting negative numbers

 if(joyX<-100 || joyX>100 || joyY<-100 || joyY>100)     return;      // commmunication error
 
// Your code here ...
//   Serial.print("Joystick position:  ");
//   Serial.print(joyX);  
//   Serial.print(", ");  
//   Serial.println(joyY);

  biasY = map(joyY, -100, 100, -255, 255); //map joystick value to angle tilt
  biasY = abs(biasY);
  joyY2 = joyY;
  Xleft = map(joyX, -100, 0, 255, 0); //map left and right turn to max PWM signal
  Xleft = constrain(Xleft, 0, 255);
  
  Xright = map(joyX, 0, 100, 0, 255);
  Xright = constrain(Xright, 0, 255);

//  Serial.println(biasY);
//  Serial.println(Xleft);
//  Serial.println(Xright);
}

void pidCompute(){ // Compute PID
  
  SetpointY = Setpoint + biasY; // add tilt to setpoint with biasY
  //Serial.println(SetpointY);
  
  InputY = roll; //input to PID is the roll value
  myPIDY.Compute();
  
  motor1PWM = abs(OutputY); // Output is converted into PWM signal
  motor1PWM = map(motor1PWM, 0, 255, motor1Start, 255); //map and constrain to set values
  motor1PWM = constrain(motor1PWM, 0, 255);

  motor2PWM = abs(OutputY);
  motor2PWM = map(motor2PWM, 0, 255, motor2Start, 255);
  motor2PWM = constrain(motor2PWM, 0, 255);

//      Serial.print("InputY ");
//      Serial.print(InputY);
//      Serial.print(" OutputY ");
//      Serial.print(OutputY);
//      Serial.print(" motor1PWM ");
//      Serial.print(motor1PWM);
//      Serial.println();
}

void motorSetup(){
    // set all the motor control pins to outputs
    pinMode(enA, OUTPUT);
    pinMode(enB, OUTPUT);
    pinMode(in1, OUTPUT);
    pinMode(in2, OUTPUT);
    pinMode(in3, OUTPUT);
    pinMode(in4, OUTPUT);
}

void motorControl(){ // set direction of motors based on tilt
  // Motor Control
  if (joyY2 < 0){
    //motor A
    digitalWrite(in1, LOW);
    digitalWrite(in2, HIGH);
    //motor B
    digitalWrite(in3, LOW);
    digitalWrite(in4, HIGH);

    analogWrite(enA, biasY - Xleft);
    analogWrite(enB, biasY - Xright);
  }

  else if (joyY2 > 0){
    //motor A
    digitalWrite(in1, HIGH);
    digitalWrite(in2, LOW);
    //motor B
    digitalWrite(in3, HIGH);
    digitalWrite(in4, LOW);

    analogWrite(enA, biasY - Xleft);
    analogWrite(enB, biasY - Xright);
  }
  
  else if (roll >= 25 && joyY2 == 0){
    //motor A
    digitalWrite(in1, HIGH);
    digitalWrite(in2, LOW);
    //motor B
    digitalWrite(in3, HIGH);
    digitalWrite(in4, LOW);

    analogWrite(enA, motor1PWM - Xleft);
    analogWrite(enB, motor2PWM - Xright);
  }

  else if (roll <= -25 && joyY2 == 0) {
    //motor A
    digitalWrite(in1, LOW);
    digitalWrite(in2, HIGH);
    //motor B
    digitalWrite(in3, LOW);
    digitalWrite(in4, HIGH);

    analogWrite(enA, motor1PWM - Xleft);
    analogWrite(enB, motor2PWM - Xright);
  }

  else {
    digitalWrite(in1, LOW);
    digitalWrite(in2, LOW);
    //motor B
    digitalWrite(in3, LOW);
    digitalWrite(in4, LOW);

    analogWrite(enA, 0);
    analogWrite(enB, 0);
  }

  //Serial.println(motor1PWM);
  //Serial.println(motor2PWM);
}

